<?php
class ModelCatalogSphinx extends Model {
	
	public function sphinxConnection() {
		return new mysqli($_SERVER['REMOTE_ADDR'], DB_USERNAME, DB_PASSWORD, DB_DATABASE, 9306, '');
	}
	
	/**
	 * Get table rows count
	 * @param array $data
	 * return int $count
	 */
	public function getTableCount($data) {
		$currentTable = $data['currentTable'];
		$query = $this->db->query("SELECT COUNT(*) AS count FROM " . DB_PREFIX . $data['tableNames'][$currentTable]);
		$count = (int)$query->row['count'];
		
		return $count;	
	}
	
	/**
	 * Get table data
	 * @param array $data - tableNames, currentTable, offset and limit
	 * return array
	 */
	public function getTableData($data = array()) {
		
		$currentTable = $data['currentTable'];
		
		$sql = "SELECT * FROM " . DB_PREFIX . $data['tableNames'][$currentTable];
		
		//Change query if table is product
		if ($data['tableNames'][$currentTable] == 'product') {
			
			$sql = "SELECT pd.product_id, pd.product_id AS product_id_attr, name, description, model, sku, ean, jan, isbn, mpn, price,
					 quantity, stock_status_id, p.status AS status, rating, pd.language_id, date_available";
			$sql .= " FROM product p"; 
			$sql .= " LEFT JOIN " . DB_PREFIX . "product_description pd ON (p.product_id = pd.product_id)";
			$sql .= " LEFT JOIN " . DB_PREFIX . "review r ON (r.product_id = p.product_id)";
		}
		
		if (isset($data['offset']) || isset($data['limit'])) {
			if ($data['offset'] < 0) {
				$data['offset'] = 0;
			}
		
			if ($data['limit'] < 1) {
				$data['limit'] = 100;
			}
				
			$sql .= " LIMIT " . (int)$data['offset'] . "," . (int)$data['limit'];
		}
		
		$query = $this->db->query($sql);
		
		return $query->rows;
		
	}
	
	public function insertDataToRtIndex($data = array()) {
		
		$currentTable = $data['currentTable'];
		
		if($data['offset'] == 0) {
			$this->session->data[$data['tableNames'][$currentTable].'_total'] = $this->getTableCount($data);
		}
		
		$results = $this->getTableData($data);
		
		switch($data['currentTable']) {
			case 1: 
				$idx = $this->rtProduct($results);
				break;
				
			case 2: 
				$idx = $this->rtSphinxSuggestions($results);
				break;
				
			case 3: 
				$idx = $this->rtCategoryDescription($results);
				break;	
		}
		
		$json = array(
				'currentTable' => $currentTable,
				'tableCount' => count($data['tableNames']),
				'offset' => $data['offset']+$idx+1,
				'limit' => $data['limit'],
				'total' => $this->session->data[$data['tableNames'][$currentTable].'_total']
		);
		
		return $json;
		
	}
	
	public function rtProduct($results) {
		return false;
		$this->load->model('catalog/product');
		
		$timeLimit = (int)ini_get('max_execution_time')-10;
		$timeStart = microtime(true);
	
		if(!is_array($results) || empty($results)) {
			return;
		}
		
		$dbc = $this->sphinxConnection();
		
		foreach($results as $idx => &$res) {
				
			if (microtime(true) - $timeStart >= $timeLimit) {
				break;
			}
			
			$categories_filter = implode(',', $this->model_catalog_product->getProductCategories($res['product_id_attr']));
			$store_filter = implode(',', $this->model_catalog_product->getProductStores($res['product_id_attr']));
			
			$sql = "INSERT INTO main (`id`, `product_id_attr`, `name`, `description`, `model`, `sku`, `ean`, `jan`, `isbn`, `mpn`, `price`, 
					`quantity`, `stock_status_id`, `status`, `rating`, `language_id`, `date_available`, `categories_filter`, `store_filter`) 
					VALUES ('".$res['product_id_attr'].$res['language_id']."',
							'".$res['product_id_attr'].$res['language_id']."',
							'".$this->db->escape($res['name'])."',
							'".$this->db->escape($res['description'])."',
							'".$this->db->escape($res['model'])."',
							'".$this->db->escape($res['sku'])."',
							'".$this->db->escape($res['ean'])."',
							'".$this->db->escape($res['jan'])."',
							'".$this->db->escape($res['isbn'])."',
							'".$this->db->escape($res['mpn'])."',
							'".$res['price']."',
							'".$res['quantity']."',
							'".$res['stock_status_id']."',									
							'".$res['status']."',
							'".$res['rating']."',
							'".$res['language_id']."',
							'".strtotime($res['date_available'])."',
							(".$categories_filter."),
							(".$store_filter.")
					)";
				
			$dbc->query($sql);
				
			if($dbc->error) {
				$this->log->write('Sphinx main_rt insert error: ' . $dbc->error);
			}
		}
	
		return $idx;
	
	}
	
	public function rtSphinxSuggestions($results){
		return false;
		$timeLimit = (int)ini_get('max_execution_time')-10;
		$timeStart = microtime(true);
	
		if(!is_array($results) || empty($results)) {
			return;
		}
	
		$dbc = $this->sphinxConnection();
		
		foreach($results as $idx => &$res) {
	
			if (microtime(true) - $timeStart >= $timeLimit) {
				break;
			}
	
			$sql = "INSERT INTO suggestions (`id`, `sphinx_suggestions_id`, `keyword`, `trigrams`, `freq`)
					VALUES (".$res['id'].",
							".$res['id'].",
							'".$res['keyword']."',
							'".$res['trigrams']."',
							".$res['freq']."
					)";
	
			$dbc->query($sql);
	
			if($dbc->error) {
				$this->log->write('Sphinx suggestions_rt insert error: ' . $dbc->error);
			}
		}
	
		return $idx;
	
	}
	
	public function rtCategoryDescription($results){
		return false;
		$timeLimit = (int)ini_get('max_execution_time')-10;
		$timeStart = microtime(true);
		
		if(!is_array($results) || empty($results)) {
			return;
		}
		
		$dbc = $this->sphinxConnection();
		
		foreach($results as $idx => &$res) {
			
			if (microtime(true) - $timeStart >= $timeLimit) {
				break;
			}
				
			$sql = "INSERT INTO categoryinfo (`id`, `category_id`, `language_id`, `name`) 
					VALUES (".$res['category_id'].$res['language_id'].",
							".$res['category_id'].$res['language_id'].",
							".$res['language_id'].", 									
							'".$this->db->escape($res['name'])."'
					)";
			
			$dbc->query($sql);
			
			if($dbc->error) {
				$this->log->write('Sphinx categoryinfo_rt insert error: ' . $dbc->error);
			}
		}
		
		return $idx;
		
	}
	
}