<?php 
require_once DIR_SYSTEM .'library/sphinx/sphinxapi.php';

class ControllerModuleSphinxhelper extends Controller { 

	private $LENGTH_THRESHOLD = 1;
	private $LEVENSHTEIN_THRESHOLD = 2;
	private $TOP_COUNT = 10;
	
	/**
	 * Sphinx initialization
	 * @return mixed - false on connection error, sphinx object on success connection
	 */
	public function initSphinx() {
  	
  		$sphinxClient = new SphinxClient();
  		$sphinxClient->SetServer($this->config->get('sphinx_search_server'), $this->config->get('sphinx_search_port'));
  		$sphinxClient->SetConnectTimeout(1);
  		$sphinxClient->_mbenc = "UTF-8";
		$sphinxClient->ResetFilters();
  		
		if(!$sphinxClient->status()) {
			return false;
		}
		
  		return $sphinxClient;
  	}
		
	/**
	 * Sphinx Search
	 * @param array $data - searchQuery, limit
	 * @return array - sphinx search status, all ids matching the search criteria
	 */
	public function sphinxSearch($data) {
  	
		$sphinxClient = $this->initSphinx();
		
		$categoryFilter = array();
		$categorySubFilter = false;
		
		if(isset($this->request->get['category_id'])) {
			$categoryFilter = array($this->request->get['category_id']);
		} elseif(isset($this->request->get['filter_category_id'])) {
			$categoryFilter = array($this->request->get['filter_category_id']);
		}
		
		if(isset($this->request->get['sub_category'])) {
			$categoryFilter = true;
		} elseif(isset($this->request->get['filter_sub_category'])) {
			$categorySubFilter = false;
		}
		
		if(isset($this->request->get['viewAll'])) {
			//Default index to search
			$index = 'productinfoextended';
		
			//Set matching mode
			$sphinxClient->SetMatchMode(SPH_MATCH_EXTENDED);
			 
			//Set how to rank the weighted values
			$sphinxClient->SetRankingMode(SPH_RANK_SPH04);

			$data['searchQuery'] = $this->properStarQuery($data['searchQuery']);	

		} else {
			$index = 'main';
		
			$fieldWeights = $this->config->get('sphinx_weights');
			foreach ($fieldWeights as $key => $val) {
				$weights[$key] = (int)$val;
			} 
			
			//Set field weights
			$sphinxClient->SetFieldWeights($weights);
			
			//Set matching mode
			$sphinxClient->SetMatchMode((int)$this->config->get('sphinx_match_mode'));
						
			//Set how to rank the weighted values
			$sphinxClient->SetRankingMode((int)$this->config->get('sphinx_ranking_mode'));
			
			//Set sort mode
			if((int)$this->config->get('sphinx_search_sort')) {
				$sortMode = (int)$this->config->get('sphinx_sort_mode');
				$sortAttrArr = $this->config->get('sphinx_sort_attr_val');
				$sphinxClient->SetSortMode($sortMode, $sortAttrArr[$sortMode]);
			}
			
		}
		
		//Set quantity filter
		$quantity = (int)$this->config->get('sphinx_search_products_quantity');
		$sphinxClient->SetFilterRange('quantity', $quantity, 2147000000); 
		
		//Set product status filter
		if((int)$this->config->get('sphinx_search_product_status') == 0) {
			$sphinxClient->setFilter('status', array(1));
		}
		
		//Set categories filter
		if(!empty($categoryFilter)) {
			$filter = $this->getFilters($categoryFilter, $categorySubFilter);
			$sphinxClient->setFilter("categories_filter", $filter);
			
			//Change index to productinfoextended
			$index = 'productinfoextended';
		}
		
		//Set store filter
		$storeID = (int)$this->config->get('store_id');
		$sphinxClient->SetFilter('store_filter', array($storeID)); 
		
		//Change the sort mode if user has selected the 'sort by' option in frontend
		if(isset($this->request->get['sort']) && isset($this->request->get['order']) && $this->request->get['sort'] != 'p.sort_order') {
			$sort = str_replace(array('pd.', 'p.'), '', $this->request->get['sort']);
			$sphinxClient->SetSortMode(SPH_SORT_EXTENDED, $sort.' '.$this->request->get['order']);
		}
						
		//Set the maximum number of search results to return
		$page = (isset($this->request->get['page'])) ? $this->request->get['page'] : 1;
		$offset = ($page - 1) * $data['limit'];
		$sphinxClient->SetLimits($offset, (int)$data['limit'], 10000);				
		
		//Return the results as an array
		$sphinxClient->SetArrayResult(true);

		$searchResults = $sphinxClient->Query($data['searchQuery'], $index);
		
		$searchInfo = array();

		if ($searchResults === false) {
			$searchInfo['status'] = 'failed';
			$this->log->write('Sphinx search failed: ' . $sphinxClient->GetLastError());
		} else {
			if ($sphinxClient->GetLastWarning()) {
				$searchInfo['status'] = 'warning';
				$this->log->write('Sphinx search warning: ' . $sphinxClient->GetLastWarning());
			} else {
				$searchInfo['status'] = 'good';
			}
			
			$searchInfo['result_total'] = $searchResults['total'];
			$searchInfo['result_found'] = $searchResults['total_found'];

			$matches = array();
			$ids = array();
			
			if (isset($searchResults["matches"]) && is_array($searchResults["matches"])) {
				foreach ($searchResults["matches"] as $docinfo) {
					array_push($ids, mysql_real_escape_string($docinfo['id']));
				}
			}

			$searchInfo['matches'] = $matches;
		}

		return array(
				'searchInfo' => $searchInfo,
				'ids' => $ids
		);
	}
	
	/**
	 * Sphinx Search Categories
	 * @param array $data - searchQuery, limit
	 * @return array - sphinx search status, all ids matching the search criteria
	 */
	public function sphinxSearchCategories($data) {
		 
		$sphinxClient = $this->initSphinx();

		$index = 'main';

		//Set sort mode
		if((int)$this->config->get('sphinx_search_sort')) {
			$sortMode = (int)$this->config->get('sphinx_sort_mode');
			$sortAttrArr = $this->config->get('sphinx_sort_attr_val');
			$sphinxClient->SetSortMode($sortMode, $sortAttrArr[$sortMode]);
		}
				
		//Set store filter
		$storeID = (int)$this->config->get('store_id');
		$sphinxClient->SetFilter('store_filter', array($storeID));
	
		//Set categories filter - this is for the categories pages only
		$sphinxClient->setFilter("categories_filter", array($data['filter_category_id']));
			
		//Set quantity filter
		$quantity = (int)$this->config->get('sphinx_category_product_quantity');
		$sphinxClient->SetFilterRange('quantity', $quantity, 2147000000);
			
		//Set product status filter
		if((int)$this->config->get('sphinx_category_product_status') == 0) {
			$sphinxClient->setFilter('status', array(1));
		}
		
		//Change the sort mode if user has selected the 'sort by' option in frontend
		if(isset($this->request->get['sort']) && isset($this->request->get['order']) && $this->request->get['sort'] != 'p.sort_order') {
			$sort = str_replace(array('pd.', 'p.'), '', $this->request->get['sort']);
			$sphinxClient->SetSortMode(SPH_SORT_EXTENDED, $sort.' '.$this->request->get['order']);
		}
	
		//Set the maximum number of search results to return
		$page = (isset($this->request->get['page'])) ? $this->request->get['page'] : 1;
		$offset = ($page - 1) * $data['limit'];
		$sphinxClient->SetLimits($offset, (int)$data['limit'], 10000);
	
		//Return the results as an array
		$sphinxClient->SetArrayResult(true);
	
		$searchResults = $sphinxClient->Query('', $index);

		$searchInfo = array();
	
		if ($searchResults === false) {
			$searchInfo['status'] = 'failed';
			$this->log->write('Sphinx search failed: ' . $sphinxClient->GetLastError());
		} else {
			if ($sphinxClient->GetLastWarning()) {
				$searchInfo['status'] = 'warning';
				$this->log->write('Sphinx search warning: ' . $sphinxClient->GetLastWarning());
			} else {
				$searchInfo['status'] = 'good';
			}
		
			$searchInfo['result_total'] = $searchResults['total'];
			$searchInfo['result_found'] = $searchResults['total_found'];
	
			$matches = array();
			$ids = array();
			
			if (isset($searchResults["matches"]) && is_array($searchResults["matches"])) {
				foreach ($searchResults["matches"] as $docinfo) {
					array_push($ids, mysql_real_escape_string($docinfo['id']));
				}
			}
	
			$searchInfo['matches'] = $matches;
		}
	
		return array(
				'searchInfo' => $searchInfo,
				'ids' => $ids
		);
	}
	
	/**
	 * Get Sphinx Suggestions
	 * @param string $search - the searched term
	 * @return string - closest match
	 */
	public function getSphinxSuggestion($search) {
		
		$sphinxClient = $this->initSphinx();
		$searchResults = $sphinxClient->Query($search, 'main');
		
		$query = trim($search);
		$words = array();
		$br = 0;
		
		if(isset($searchResults['words'])) {
			foreach($searchResults['words'] as $k=>$v) {
				$words[$br]['keyword'] = $k;
				$words[$br]['docs'] = $v['docs'];
				$br++;
			}
			
			$suggestion = $this->makePhaseSuggestion($words, $query);

			return $suggestion;
		} else {
			return false;
		}
	}
		
	public function buildTrigrams($keyword)	{
		
		mb_internal_encoding("UTF-8");
		
		$t = "__" . $keyword . "__";
		
		$trigrams = "";
		for ($i = 0; $i < mb_strlen($t) - 2; $i++) {
			$trigrams .= mb_substr($t, $i, 3) . " ";
		}

		return $trigrams;
	}
	
	public function makeSuggestion($keyword) {
		
		$trigrams = $this->buildTrigrams($keyword);
		
		$query = "\"$trigrams\"/1";
		$len = strlen($keyword);
	
		$delta = $this->LENGTH_THRESHOLD;
		$sphinxClient = $this->initSphinx();
		$sphinxClient->SetMatchMode(SPH_MATCH_EXTENDED2);
		$sphinxClient->SetRankingMode(SPH_RANK_WORDCOUNT);
		$sphinxClient->SetFilterRange("len", $len-$delta, $len+$delta );
		$sphinxClient->SetSelect("*, @weight+$delta-abs(len-$len) AS myrank");
		$sphinxClient->SetSortMode(SPH_SORT_EXTENDED, "myrank DESC, @weight DESC, freq DESC");
		$sphinxClient->SetLimits(0, $this->TOP_COUNT);
		$sphinxClient->SetArrayResult(true);
	
		$res = $sphinxClient->Query($query, 'suggestions');
	
		if (!$res || !isset($res["matches"])) {
			return false;
		}
	
		// further restrict trigram matches with a sane Levenshtein distance limit
		foreach ($res["matches"] as $match) {
			$suggested = $match["attrs"]["keyword"];
			
			if (levenshtein($keyword, $suggested) <= $this->LEVENSHTEIN_THRESHOLD) {
				return $suggested;
			}
		}
	
		return $keyword;
	}
	
	public function makePhaseSuggestion($words, $query) {
		
		$suggested = array();
		$llimf = 0;
		$i = 0;
		foreach ($words  as $key => $word) {
			if ($word['docs'] != 0) {
				$llimf +=$word['docs'];$i++;
			}
		}
	
		if(($i * $i) > 0) {
			$llimf = $llimf / ($i * $i);
		}
	
		foreach ($words  as $key => $word) {
			if ($word['docs'] == 0 | $word['docs'] < $llimf) {
				$mis[] = $word['keyword'];
			}
		}
	
		if (isset($mis) && count($mis) > 0) {
			foreach ($mis as $m) {
				$re = $this->makeSuggestion($m);
				
				if ($re) {
					if($m!=$re) {
						$suggested[$m] = $re; 
					}
				}
			}
			
			if(count($words) == 1 && empty($suggested)) {
				return false;
			}
	
			$phrase = explode(' ', $query);
			foreach ($phrase as $k => $word) {
				if (isset($suggested[strtolower($word)]))
					$phrase[$k] = $suggested[strtolower($word)];
			}
	
			$phrase = implode(' ', $phrase);
	
			return $phrase;
		} else {
			return false;
		}
	}
	
	/**
	 * Get all categories to filter by
	 * @return array of ids
	 */
	public function getFilters($categoryFilter, $categorySubFilter) {
		
		if($categorySubFilter) {
			$subCategories = $this->model_catalog_category->getCategories($categoryFilter[0]);
			if(!empty($subCategories)) {
				foreach($subCategories as $arr) {
					$categories[] = (int)$arr['category_id'];
				}
			}
		}

		return (isset($categories)) ? $categories : $categoryFilter;
	}
	
	public function properStarQuery($searchQuery) {
		$query = '';
		$properQuery = explode(' ',$searchQuery);
		foreach($properQuery as $word) {
			$query .= '*'.$word.'* ';
		}
		
		return $query;
	}
	
	public function sphinxMatchModes() {
	
		return array(
				'0' => 'SPH_MATCH_ALL',
				'1' => 'SPH_MATCH_ANY',
				'2' => 'SPH_MATCH_PHRASE',
				'3' => 'SPH_MATCH_BOOLEAN',
				'4' => 'SPH_MATCH_EXTENDED',
				'5' => 'SPH_MATCH_FULLSCAN',
				'6' => 'SPH_MATCH_EXTENDED2',
		);
	
	}
	
	public function sphinxSortModes() {
	
		return array(
				'0' => array( 'mode' => 'SPH_SORT_RELEVANCE'),
				'1' => array( 'mode' => 'SPH_SORT_ATTR_DESC', 'attrs' => $this->sphinxSortAttrs()),
				'2' => array( 'mode' => 'SPH_SORT_ATTR_ASC', 'attrs' => $this->sphinxSortAttrs()),
				'3' => array( 'mode' => 'SPH_SORT_TIME_SEGMENTS'),
				'4' => array( 'mode' => 'SPH_SORT_EXTENDED'),
				'5' => array( 'mode' => 'SPH_SORT_EXPR')
		);
	
	}
	
	public function sphinxRankingModes() {
	
		return array(
				'0' => 'SPH_RANK_PROXIMITY_BM25',
				'1' => 'SPH_RANK_BM25',
				'2' => 'SPH_RANK_NONE',
				'3' => 'SPH_RANK_WORDCOUNT',
				'4' => 'SPH_RANK_PROXIMITY',
				'5' => 'SPH_RANK_MATCHANY',
				'6' => 'SPH_RANK_FIELDMASK',
				'7' => 'SPH_RANK_SPH04',
				'8' => 'SPH_RANK_EXPR',
				'9' => 'SPH_RANK_TOTAL',
		);
	
	}
	
	public function sphinxSortAttrs() {
	
		return array(
				'name' => 'name',
				'description' => 'description',
				'model' => 'model',
				'sku' => 'sku',
				'ean' => 'ean',
				'jan' => 'jan',
				'isbn' => 'isbn',
				'mpn' => 'mpn',
				'price' => 'price'
		);
	
	}

}
?>